# This software is covered by the Zope Public License
# It can be found at: http://www.zope.org/Resources/ZPL
#
# Copyright ActiveState Tool Corp.
# Andy McKay, Jan 2002.
# AndyM@ActiveState.com

# usual imports
from Globals import Persistent, DTMLFile
import Acquisition
import OFS.SimpleItem
import AccessControl.Role
from DateTime import DateTime

# specific imports
import re
import win32com.client
from urllib import quote
from pythoncom import CoInitializeEx, COINIT_MULTITHREADED

addPassportForm = DTMLFile('dtml/Add', globals())
cookieRe = re.compile(' (\w+)=(.*?);')
cookiesFromMS = ['MSPAuth', 'MSPProf']

# this is some of the passport data available
passportProfileKeys = [ 
'Accessibility', # (VT_I1) 1 Indicates whether accessibility features should be enabled on participating sites for this user. 0=no; 1=yes. See details. Must be 0 or 1. 
'BDay_precision', # * Integer (VT_I1) 1 Defines the precision of the Birthdate attribute. See details. 0, 1, 2, 3 or Null 
'Birthdate', # Long (VT_DATE) 8 Contains the user's birth year or birthdate; only valid dates since 12/30/1899. See details. See details. 
'City', # Long (VT_I4) 4 GeoID that maps to the user's city. See details. Must be a valid GeoID. 
'Country', # Text (VT_BSTR) 2 ISO 3166 country code for the user's country/region. See details. See details. 
'Directory', # Long (VT_I4) 4 Not implemented. Reserved for future use. N/A 
'FirstName', # Varying length UNICODE string 128 Specifies the first name of the user. See details. 
'Flags', # * Long (VT_I4) 4 Reserved network flags. See details. N/A 
'Gender', # Text (VT_BSTR) 1 Gender of user. See details. Must be Null, M, or F. 
'Lang_Preference', # Long (VT_I4) 4 LCID of the user's preferred language. See details. See details. 
'LastName', # Varying length UNICODE string 128 Specifies the last name of the user. See details. 
'MemberIDHigh', #* Long (VT_I4) 4 The upper portion of the 64-bit Microsoft Passport Unique ID (PUID) primary key that uniquely identifies a .NET Passport user. The PUID maps one-to-one to a user's sign-in name. See details. N/A 
'MemberIDLow', #* Long (VT_I4) 4 The lower portion of the 64-bit PUID primary key that uniquely identifies a .NET Passport user. The PUID maps one-to-one to a user's sign-in name. See details. N/A 
'MemberName', # Text 129 A legacy attribute that no longer contains sign-in name, but can be used to determine a user's domain. See details. See details. 
'Nickname', # Text (VT_BSTR) 60 Friendly name the user would like to be greeted by. See details. See details. 
'Occupation', # One ASCII character (1 byte) 1 Indicates the occupation of the user. See details. 
'PreferredEmail', # Text (VT_BSTR) 129 The user's e-mail address. See details. See details. 
'PostalCode', # Text (VT_BSTR) 15 Stores the postal code for the United States and other countries/regions, where appropriate. See details. See details. 
#ProfileVersion', #** Long (VT_I4) 4 Represents the version of the user's core profile. See details. N/A 
'Region', # Long (VT_I4) 4 GeoID that maps to the region within the user's country/region. See details. Must be a valid GeoID.  
'TimeZone', # Short signed integer (two bytes) 2 Indicates the user's time zone. See details. 
#Wallet* 
]


def addPassport(self, id, title='', REQUEST=None):
    """ Create a Passport Object """
    i=Passport(id,title)
    self._setObject(id,i)
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)

class Passport(OFS.SimpleItem.Item, Persistent, Acquisition.Explicit, AccessControl.Role.RoleManager):
    """ Gives Zope access to Passport."""
    meta_type="Passport"

    manage_options=(
        {'label':'Properties', 'action':'manage_main'},
        {'label':'Security', 'action':'manage_access'},
        )

    __ac_permissions__=(
        ('View management screens', ('manage_main',)),
        ('Change Passport', ('manage_edit','refresh')),
        ('Use Passport', ('__call__', 'getProfile', 'getLogo', 'getExpressPurchase', 'hasTicket', 'logOut')),
        )

    manage_main = DTMLFile('dtml/Main', globals())

    def __init__(self, id, title=''):
        ''' Initialise, set the title and add in the COM objects '''
        self.id=id
        self.manage_edit(title)
        self.refresh()

    # protected by the 'Change Passport permission'
    def manage_edit(self, title='', REQUEST=None):
        """ Edit Passport """
        self.title=title
        if REQUEST is not None:
            return self.manage_main(self, REQUEST,
                manage_tabs_message="Changed."
                )

    # protected by the 'Change Passport permission'
    def refresh(self, REQUEST=None):
        ''' Add objects in '''
        self._oAdm()
        self._oMgr()
        if REQUEST is not None:
            return self.manage_main(self, REQUEST,
                    manage_tabs_message="Refreshed."
                    )

    # protected by the 'Use Passport permission'
    def getProfile(self):
        ''' Returns the profile as a dict '''
        prof = {}
        p =self._oMgr().Profile
        for key in passportProfileKeys:
            prof[key] = p(key)
        return prof

    # protected by the 'Use Passport permission'
    def getLogo(self):
        """ Returns some html """
        # <%=oMgr.logoTag(Server.URLEncode("http://"&Request.ServerVariables("SERVER_NAME")&Request.ServerVariables("SCRIPT_NAME") & "?lang=" & langId),10000,FALSE,"nada",langId, bSecure)%>
        return self._oMgr().logoTag()

    # protected by the 'Use Passport permission'
    def getExpressPurchase(self):
        ''' Gets the express purchase button '''
        return "https://wallet.passport.com/ppsecure/get.asp?mid=1&ru=%s" % quote(self.REQUEST['SERVER_URL'])

    # protected by the 'Use Passport permission'
    def hasTicket(self):
        """ Checks to see if a user has a ticket """
        return self._oMgr().HasTicket

    # protected by the 'Use Passport permission'
    def logOut(self):
        ''' Log Out by deleting cookies '''
        self._deleteCookies()

    ################################
    # These are all protected by the underscore and 
    # are internal methods for manipulating the COM
    # objects...

    def _addCOM(self, dispatch):
        ''' Actually add the COM object in '''
        CoInitializeEx(COINIT_MULTITHREADED)
        return win32com.client.Dispatch(dispatch)

    def _addCOMObjects(self, name, dispatch):
        ''' Put it in a volatile attr '''
        obj = self._addCOM(dispatch)
        setattr(self, '_v_%s' % name, obj)

    def _getReqData(self):
        ''' Gets all the cookie data we might want '''
        t = self.REQUEST.form.get('t', '')
        p = self.REQUEST.form.get('p', '')
        ca = self.REQUEST.cookies.get('MSPAuth', '')
        cp = self.REQUEST.cookies.get('MSPProf', '')
        cpc = self.REQUEST.cookies.get('MSPProfC', '')
        return (t, p, ca, cp, cpc)

    def _initCall(self):
        ''' This initialises the object correctly, note
        we only need to do OnStartPageManual once per
        request '''
        if type(self.REQUEST) != type(''):
            p = self.REQUEST.get('PassportInit', None)
            if p is None:
                (t, p, ca, cp, cpc) = self._getReqData()
                result = self._v_Mgr.OnStartPageManual(t, p, ca, cp, cpc, [])
                if result is not None:
                    self._resultCookies(result)
                self.REQUEST.set('PassportInit', 1)

    def _deleteCookies(self):
        ''' Delete the cookies... '''
        resp = self.REQUEST.RESPONSE
        for c in cookiesFromMS:
            resp.expireCookie(c)

    def _resultCookies(self, res):
        ''' Set the cookies... '''
        # set the cookie domain for this host, alter as 
        dmn = self.REQUEST['HTTP_HOST']
        resp = self.REQUEST.RESPONSE
        # sets an expiry of one year for the passport cookie
        exp = str((DateTime('GMT')+365).rfc822())
        for r, c in map(None, res, cookiesFromMS):
            m = cookieRe.search(r)
            if m is not None:
                resp.setCookie(c, m.group(2), domain=dmn, path='/', expires=exp)

    def _oAdm(self):
        ''' Get the admin object '''
        if not hasattr(self, '_v_Adm'): 
            self._addCOMObjects('Adm', 'Passport.Admin')
        return self._v_Adm

    def _oMgr(self):
        ''' Get the Manager object, use this so the initCall is 
        done and the object is initialised correctly '''
        if not hasattr(self, '_v_Mgr'): 
            self._addCOMObjects('Mgr', 'Passport.Manager')
        self._initCall()
        return self._v_Mgr
